//
//  PSFPlayer.m
//  PSFPlay
//
//  Created by tmkk on Sat Jul 24 2004.
//  Copyright (c) 2004 tmkk. All rights reserved.
//

#import "PSFPlayer.h"
#import "PSFPController.h"

@implementation PSFPlayer

- (id)initWithDelegate: (id)del
{
	[super init];
	delegate = [del retain];
	pause = NO;
	isPlaying = NO;
	playMode = 0;
	return self;
}

- (void)dealloc
{
	[delegate release];
	[super dealloc];
}

- (void)playPSF: (NSString *)psf
{
	task = [[NSTask alloc] init];
	[task setStandardOutput: [NSPipe pipe]];
	[task setStandardInput: [NSPipe pipe]];
	[task setLaunchPath: [[NSBundle mainBundle] pathForResource:@"psfplay" ofType:nil inDirectory:nil]];
	NSMutableArray *args = [NSMutableArray arrayWithObjects: @"-m", [delegate createChannelMaskforPSF], @"-v", [NSString stringWithFormat: @"%1.2f", [delegate volume]], nil];
	if([delegate reverbEnabledPSF]) {
		[args addObject: @"-r"];
		[args addObject: [NSString stringWithFormat: @"%d", [delegate reverbType]]];
		[args addObject: @"-s"];
		[args addObject: [NSString stringWithFormat: @"%d", [delegate reverbStrength]]];
	}
	[args addObject: psf];
	[task setArguments: args];
	[[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(getData:) 
        name: @"NSFileHandleReadCompletionNotification"
        object: [[task standardOutput] fileHandleForReading]];
	[[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(taskDidEnd:) 
        name: @"NSTaskDidTerminateNotification" 
        object: task];
	[[[task standardOutput] fileHandleForReading] readInBackgroundAndNotify];
	[task launch];
	playMode = 0;
	isPlaying = YES;
}

- (void)playSPC: (NSString *)spc
{
	/*
	if(isPlaying) {
		[task terminate];
	}
	*/
	task = [[NSTask alloc] init];
	[task setStandardOutput: [NSPipe pipe]];
	[task setStandardInput: [NSPipe pipe]];
	[task setLaunchPath: [[NSBundle mainBundle] pathForResource:@"spcplay" ofType:nil inDirectory:nil]];
	NSMutableArray *args = [NSMutableArray arrayWithObjects: @"-t", [delegate playTime],  @"-f", [delegate fadeOut], @"-m", [delegate createChannelMaskforSPC], @"-i", [NSString stringWithFormat: @"%d", [delegate interpolationMethod]], @"-v", [NSString stringWithFormat: @"%1.2f", [delegate volume]], nil];
	if([delegate reverbEnabledSPC]) {
		[args addObject: @"-r"];
		[args addObject: [NSString stringWithFormat: @"%d", [delegate reverbType]]];
		[args addObject: @"-s"];
		[args addObject: [NSString stringWithFormat: @"%d", [delegate reverbStrength]]];
	}
	if([delegate forcePlayTime]) [args addObject: @"-F"];
	if([delegate detectSilence]) {
		[args addObject: @"-c"];
		[args addObject: [NSString stringWithFormat: @"%d", [delegate silence]]];
	}
	[args addObject: spc];
	[task setArguments: args];
	[[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(getData:) 
        name: @"NSFileHandleReadCompletionNotification"
        object: [[task standardOutput] fileHandleForReading]];
	[[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(taskDidEnd:) 
        name: @"NSTaskDidTerminateNotification" 
        object: task];
	[[[task standardOutput] fileHandleForReading] readInBackgroundAndNotify];
	[task launch];
	playMode = 1;
	isPlaying = YES;
}

- (void)clearTask
{
	if(task) [task release];
	task = nil;
}

- (void)stop
{
	if(!isPlaying) return;
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[task terminate];
	if(pause) [self pause];
	[self clearTask];
	isPlaying = NO;
}

- (void)pause
{
	if(!pause) {
		[task suspend];
		pause = YES;
	}
	else {
		[task resume];
		pause = NO;
	}
}

- (BOOL)isPause
{
	return pause;
}

- (void)taskDidEnd: (NSNotification *)aNotification
{
	int status = [[aNotification object] terminationStatus];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[self clearTask];
	pause = NO;
	isPlaying = NO;
	//[[[task standardOutput] fileHandleForReading] closeFile];
	//[self stopProcess: [aNotification object]];
	if(status == 0) {
		//isPlaying = YES;
		[delegate playDone];
	}
}

- (void)getData: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
    if ([data length])
    {
		if(!memcmp([data bytes],"\r",1)) {
			NSArray *arr = [[[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease] componentsSeparatedByString:@" "];
			[delegate setPlayTime: [arr objectAtIndex:1]];
		}
	}
	
    if([task isRunning]) [[aNotification object] readInBackgroundAndNotify];  
}

- (void)toggleMuteChannelSPC: (int)ch
{
	if(!isPlaying || !playMode) return;
	char c[2];
	c[0] = 'm';
	c[1] = ch + 0x30;
	[[[task standardInput] fileHandleForWriting] writeData: [NSData dataWithBytes: c length: 2]];
	
}

- (void)toggleMuteChannelPSF: (int)ch
{
	if(!isPlaying || playMode) return;
	char c[2];
	c[0] = 'm';
	if(ch < 10) c[1] = ch + 0x30;
	else c[1] = ch + 0x57;
	[[[task standardInput] fileHandleForWriting] writeData: [NSData dataWithBytes: c length: 2]];
}

- (void)setVolume: (float)vol
{
	if(!isPlaying) return;
	char buf[10];
	sprintf(buf,"v%1.2f",vol);
	[[[task standardInput] fileHandleForWriting] writeData: [NSData dataWithBytes: buf length: strlen(buf)+1]];
}

@end
